<?php
/**
* @Copyright ((c) plasma-web.ru
        * @license    GPLv2 or later
        */

namespace PPF3;

\defined('PPF3_VERSION') or die;

class Form extends ppf_admin
{
    public $xml;
    public $xmlarr;
    public $items;

    public function __construct($tpl, $items, $path='')
    {
        $this->items = $items;

        if(! $path) $path = PPF3_ADMIN_DIR . 'forms/' . $tpl . '.xml';
        if (is_file($path)) {
            $this->xml = simplexml_load_file($path);
        }
    }

    public function renderForm()
    {
        $html = '';
        $heder = '<div class="ppftabslabelsbox">';
        $i = 0;
        foreach ($this->xml->fieldset as $v) {
            $heder .= '<div class="ppftabsitemlabel'.(!$i?' ppftabactiv':'').'">'.Text::_((string)$v->attributes()->label).'</div>';
            $i++;
            $html .= '<div class="ppftabsitem">';
            foreach ($v as $field) {
                $html .= $this->renderField((string)$field['name']);
            }
            $html .= '</div>';

        }
        $heder .= '</div>';
        return ($heder.$html);
    }


    public function renderFieldset($name)
    {
        $fieldset = $this->getfieldset($this->xml, $name);
        if(! $fieldset) return '';
        $html = '';

        foreach ($fieldset as $field) {
            $html .= $this->renderField((string)$field['name']);
        }
        return $html;
    }

    public function renderField($name)
    {
        if(! isset($this->XMLToArray()[$name])) return;
        
        $field = $this->XMLToArray()[$name]->attributes();

        $required = $this->get('required', $field) ? ' required' : '';
        $readonly = $this->get('readonly', $field) ? ' readonly' : '';
        $checked = $this->get('checked', $field) ? ' checked' : '';
        $class = $this->get('class', $field) ? ' class="'.$this->get('class', $field).'"' : '';
        $id = $this->get('id', $field) ? ' id="'.$this->get('id', $field).'"' : '';
        $attr = $id.$class.$required.$readonly.$checked;
        $desk = $this->get('description', $field) ? '<div class="desk">'.Text::_($this->get('description', $field)).'</div>' : '';
        $value = htmlspecialchars($this->get('value', $field),ENT_QUOTES);
        if($this->get('translate', $field)) $value = Text::_($value);

        $html = '<div class="ppffield box_'.$this->get('class', $field).'">';
        $html .= '<div class="ppffieldlabel">' . Text::_($this->get('label', $field)) . ($required ? '<span class="req">*</span>':'') .'</div>';
        $html .= '<div class="ppffieldbody">';

        $type = $this->get('type', $field);

        if ($type == 'text') {
            $html .= '<input type="text" name="ppffield['.$name.']" value="'.$value.'"'.$attr.'>' . $desk;
        } elseif ($type == 'radio') {
            $options = $this->XMLToArray()[$name];
            $i = 0;

            foreach ($options as $k=>$option) {
                $oattr = $option->attributes();
                $checked = $value == (string)$oattr['value'] ? ' checked' : '';
                $i++;

                $html .= '<label class= "lab'.$i.'"><input type="radio" name="ppffield['.$name.']" value="'.$oattr['value'].'"'.$checked.$attr.'>'.Text::_((string)$option).'</label>';
            }
            $html .= $desk;
        } elseif ($type == 'filelist') {
            $path = PPF3_PLUGIN_DIR . $this->get('directory', $field);
            if (is_dir($path)) {
                $filter = '#'.$this->get('fileFilter', $field, '(.*).jpg').'#';

                $html .= '<select name="ppffield['.$name.']"'.$attr.'>';
                if ($this->get('hide_none', $field) == 'false') {
                    $html .= '<option value="">'.Text::_('PPF_NOT_SELECTED').'</option>';
                }

                $fname  = scandir($path);
                for ($i=0, $n=count($fname); $i < $n; $i++) {
                    if (preg_match($filter, $fname[$i])) {
                        $selected = $value == $fname[$i] ? ' selected="selected"' : '';
                        $html .= '<option value="'.$fname[$i].'"'.$selected.'>'.$fname[$i].'</option>';
                    }
                }

                $html .= '</select>';
                $html .= $desk;
            }
        }
        elseif($type == 'folderlist') {
          $path = PPF3_PLUGIN_DIR . $this->get('directory', $field);
          if (is_dir($path)) {

              $html .= '<select name="ppffield['.$name.']"'.$attr.'>';
              if (! $this->get('hide_none', $field)) {
                  $html .= '<option value="">'.Text::_('PPF_NOT_SELECTED').'</option>';
              }

              $fname  = scandir($path);
              for ($i=0, $n=count($fname); $i < $n; $i++) {
                  if (is_dir($path.'/'.$fname[$i]) && $fname[$i] != '.' && $fname[$i] != '..') {
                      $selected = $value == $fname[$i] ? ' selected="selected"' : '';
                      $html .= '<option value="'.$fname[$i].'"'.$selected.'>'.$fname[$i].'</option>';
                  }
              }

              $html .= '</select>';
              $html .= $desk;
          }
        } elseif ($type == 'checkbox') {
            $html .= '<input type="checkbox" name="ppffield['.$name.']" value="'.$value.'"'.$attr.'>' . $desk;
        } elseif ($type == 'textarea') {
            $html .= '<textarea name="ppffield['.$name.']"'.$attr.'>'.$value.'</textarea>' . $desk;
        } elseif ($type == 'accesslevel') {
            if($name == 'p][freeaccess')$acs = ppf::getacs('PPF_NOBODY');
            else $acs = ppf::getacs();
            $html .= $this->listfield($name, $acs, $value);
            $html .= $desk;

        } elseif ($type == 'contentlanguage') {
            $languages = $this->getLanguages();
            $html .= $this->listfield('language', $languages, $value);
            $html .= $desk;

        }
        elseif ($type == 'version') {
            $lang = ppf::getlang();
            if ($lang == 'ru_RU') {
                $link = '<a href="http://plasma-web.ru/dev/ppf-2" target="_blank">plasma-web.ru</a>';
            } else {
                $link = '<a href="http://plasma-web.ru/en/dev/ppf-2" target="_blank">plasma-web.ru</a>';
            }

            $html .=  PPF3_VERSION . '<br />url: '.$link;

        } elseif ($type == 'spacer') {
            if($value) $html .=  '<br><br><br>'.Text::_($value).'<br>';
            $html .= $desk;

        } elseif ($type == 'list') {
            $options = $this->XMLToArray()[$name];
            $new = array();

            foreach ($options as $k=>$option) {
                $oattr = $option->attributes();
                $new[(string)$oattr['value']] = (string)$option;
            }
            $html .= $this->listfield($name, $new, $value);
            $html .= $desk;

        } elseif ($type == 'historystatus') {
            require_once(PPF3_ADMIN_DIR . 'src/model/historys.php');
            $model = new historysModel();
            $options = $model->statusfields();
            $html .= $this->listfield($name, $options, $value);
        }
        elseif($type == 'hidden') {
          $html .= '<input type="hidden" name="ppffield['.$name.']" value="'.$value.'"'.$attr.'>';
        }
        elseif($type == 'resurl') {
          $html .= ((! empty($_SERVER['HTTPS'])) ? 'https' : 'http') . '://' . $_SERVER['HTTP_HOST'] . '/index.php?option=com_ppf3&task=resurl&result=' . $value;
        }
        elseif($type == 'pslink') {
          $html .=  '<a href="'.$value.'">'.$value.'</a>' . $desk;
        }
        elseif($type == 'secret') {
          if (! $this->get('text', ppf::config())) {
            $attr .= ' disabled';
          }
          $html .= '<input type="text" name="ppffield['.$name.']" value="'.$value.'"'.$attr.'>' . $desk;
        }
        elseif($type == 'categories') {
          $tree = ppf::categories()->categoriesTree();
          $options = $this->catlist($tree);
          $html .= $this->listfieldCategory($name, $options, $value, ' class="catsel" size=20');
        }
        elseif($type == 'ppffile') {
          $html .= '<input type="hidden" name="ppffield['.$name.']" value="">';
          $html .= '<input type="file" name="ppffield['.$name.']" '.$attr.'>';
          $text = '<span style="color:red">'.Text::_('PPF_NOT_FILE').'</span>';
          if($value) {
            $path = ppf::getfolder($this->items->id) . $value;
            if(is_file($path)) {
              $text = $value . ' ('.$this->convert_bytes(filesize($path)).').';
            }
          }

          $text .= '<br>server limit: '. get_cfg_var('upload_max_filesize');
          
          $html .= '<div class="desk">'.Text::_('PPF_FILE_DNL').' '.$text.'</div>';
        }
        elseif($type == 'videourl') {
          $html .= '<input type="text" name="ppffield['.$name.']" value="'.$value.'"'.$attr.'>';
          if($value) {
            $pats = explode($_SERVER['HTTP_HOST'], $value);
            $file = JPATH_ROOT . $pats[1];
            if(! is_file($file)) $text =  '<span style="color:red">'.Text::_('PPF_NOT_FILE').'</span>';
            else {
              $filesize = filesize($file);
              $text = number_format($filesize/1024, 0, '.', ' ') . ' kB.';
            }
            $html .= '<div class="desk2">'.Text::_('PPF_FILE_DNL').' '.$text.'</div>';
          }
          $html .= $desk;
        }
        elseif($type == 'price') {
          $value = isset($this->items->price) ? $this->items->price : '';
          $html .= '<div class="fileblok"><div><input type="text" name="ppffield[price]" value="'.$value.'"'.$attr.'></div>';
          $curr = explode(',', ppf::config()->currency);
          $options = [];
          foreach($curr as $cur){
            $options[trim($cur)] = trim($cur);
          }
          $value = isset($this->items->currency) ? $this->items->currency : '';
          $html .= '<div>'.$this->listfield('currency', $options, $value) . '</div></div>';
          $html .= $desk;
        }
        elseif($type == 'currency') {
          $curr = explode(',', ppf::config()->currency);
          $options = [];
          foreach($curr as $cur){
            $options[trim($cur)] = trim($cur);
          }
          $html .= $this->listfield($name, $options, $value);
          $html .= $desk;
        }
        elseif($type == 'psbtns') {
          $db = \JFactory::getDbo();
          $db->setQuery("SELECT id, title FROM #__ppf3_ps");
          $pss = $db->loadObjectList();
          
          $name2 = 'p][psbtns';
          $value = isset($this->items->$name2) ? (array)$this->items->$name2 : array();
          
          $html .= '<input type="hidden" name="ppffield['.$name2.']" value="">';
          
          $html .= '<select name="ppffield['.$name.']" multiple class="'.$name.'" >';
          foreach ($pss as $ps) {
              $selected = '';
              if (in_array($ps->id, $value)) {
                  $selected = ' selected="selected"';
              }
              $html .= '<option value="'.$ps->id.'"'.$selected.'>'.Text::_((string)$ps->title).'</option>';
          }
          $html .= '</select>';

          $html .= $desk;
        }
        elseif($type == 'fmcategories') {
          $name2 = 'categories';
          $value = isset($this->items->$name2) ? $this->items->$name2 : '';
          
          $tree = ppf::categories()->categoriesTree();
          $options = $this->catlist($tree);
          $html .= $this->listfieldCategory2($name, $options, $value, ' multiple class="categories" size=20');
          $html .= $desk;
        }
        elseif($type == 'checkfile') {
          if($this->items && isset($this->items->{'p][ppffile'})) {
            $path = ppf::getfolder($this->items->id, false) . $this->items->{'p][ppffile'};
            $html .= $path;
          }
          elseif($this->items && isset($this->items->{'p][url'})) {
             $html .= $this->items->{'p][url'};
          }
          else $html .= Text::_('PPF_NOT_FILE');
          $html .= $desk;
        }
        elseif($type == 'getcod') {
          $html .= '<div class="codeblok"><div class="codeblok_l"><input type="text" name="ppemail"'.$attr.' autocomplete="none"><input type="hidden" name="pphost" value="'.$_SERVER['HTTP_HOST'].'"></div>';
          $html .= '<div class="codeblok_r"><input type="button" onclick="ppfadmin.plasma(this)"  value="'.Text::_('PPF_TO_PAY').'"'.$attr.'></div></div>';
          $html .= $desk;
        }
        elseif($type == 'cbr') {
          $html .= '<a href="http://www.cbr.ru/scripts/XML_daily.asp" target="_blank">' . Text::_('PPF_CBR') . '</a>';
          $html .= $desk;
        }
        elseif ($type == 'entercode') {
          $html .= '<input type="text" name="ppffield['.$name.']" value="'.$value.'"'.$attr.'>';
          $timepats = $this->secToArray(intval(ppf::gl('lang') - time()));
          $html .= '<div class="desk">'.Text::_('PPF_FINISH') . ' ' . ($timepats['month'] ? $timepats['month'] . ' ' . Text::_('PPF_MONTHS') . ', ' : '') . $timepats['days'] . ' ' . Text::_('PPF_DAYS') . ', ' . ($timepats['minutes'] ? $timepats['hours'] . ' ' . Text::_('PPF_HOURS') . ', ' : '') . ($timepats['minutes'] ? $timepats['minutes'] .' ' . Text::_('PPF_MINUTES') . ' ' : '') . '</div>';
        }

        $html .= '</div>';
        $html .= '</div>';

        return $html;
    }
    
    public function catlist($tree, $disabled=0) {
      static $i = 0;
      static $opt=[];
      $i++;
      $d = '';
      $ii=$i;
      while($ii){
        $d .= ' — ';
        $ii--;
      }
      foreach($tree as $cat) {
        $dis = $disabled;
        if($this->items && $cat->id == $this->items->id) {
          $dis = 1;
        }
        $opt[$cat->id] = array($d.$cat->title, $dis);
        if($cat->child) {
          $this->catlist($cat->child, $dis);
          $i--;
        }
      }
      return $opt;
    }

    public function get($name, $obj, $def='')
    {
        $obj = (object) $obj;
        return isset($obj->$name) ? (string) $obj->$name : $def;
    }

    protected function XMLToArray()
    {
        if (! $this->xmlarr) {
            $this->bind($this->xml);
        }
        return $this->xmlarr;
    }

    protected function bind($xml)
    {
        foreach ($xml as $k => $v) {
            if (is_object($v) && ($k != 'field')) {
                $this->bind($v);
            } elseif ($k == 'field' && isset($v['name'])) {
                $name = (string)$v['name'];
                $def = isset($v['default'])?(string)$v['default']:'';
                if(! isset($v['value'])) {
                  $v['value'] = isset($this->items->$name) ? $this->items->$name : $def;
                }
                $this->xmlarr[$name]=$v;
            }
        }
    }

    protected function getfieldset($xml, $name)
    {
        foreach ($xml as $k => $v) {
            if ($v->getName() == 'fieldset' && (string)$v['name'] == $name) {
                return $v;
            }
        }
    }

    protected function listfield($name, $options, $sel, $atr='')
    {
        if($atr) $html = '<select name="ppffield['.$name.']"'.$atr.' >';
        else $html = '<select name="ppffield['.$name.']" class="'.$name.'" >';

        foreach ($options as $k => $option) {
            $selected = '';
            if ($sel == $k) {
                $selected = ' selected="selected"';
            }
            $html .= '<option value="'.$k.'"'.$selected.'>'.Text::_((string)$option).'</option>';
        }

        $html .= '</select>';
        return $html;
    }
    
    protected function listfieldCategory($name, $options, $sel, $atr='')
    {
        if($atr) $html = '<select name="ppffield['.$name.']"'.$atr.' >';
        else $html = '<select name="ppffield['.$name.']" class="'.$name.'" >';

        foreach ($options as $k => $option) {
            $selected = '';
            if ($sel == $k) {
                $selected = ' selected="selected" style="background:#eaf3ff"';
            }
            if($option[1]) {
              $selected = ' style="background:#f7d4ec" disabled';
            }
            $html .= '<option value="'.$k.'"'.$selected.'>'.Text::_((string)$option[0]).'</option>';
        }

        $html .= '</select>';
        return $html;
    }
    protected function listfieldCategory2($name, $options, $sel, $atr='')
    {
        $html = '<select name="ppffield['.$name.']"'.$atr.' >';
        
        $valarr = explode(',', $sel);
        foreach ($options as $k => $option) {
            $selected = '';
            if (in_array($k, $valarr)) {
                $selected = ' selected="selected"';
            }
            $html .= '<option value="'.$k.'"'.$selected.'>'.Text::_((string)$option[0]).'</option>';
        }

        $html .= '</select>';
        return $html;
    }
    
    protected function convert_bytes($size)
    {
    	$i = 0;
    	while (floor($size / 1024) > 0) {
    		++$i;
    		$size /= 1024;
    	}
     
    	$size = str_replace('.', ',', round($size, 1));
    	switch ($i) {
    		case 0: return $size .= ' B';
    		case 1: return $size .= ' K';
    		case 2: return $size .= ' M';
        case 3: return $size .= ' G';
    	}
      

    }
    
    public static function secToArray($secs)
    {
      $res = array();
      $secs = $secs < 0 ? 0 : $secs;
      
      $res['month'] = floor($secs / (30*86400));
      
      if($res['month'] > 1) {
        $secs = $secs % (30*86400);
      }
      else {
        $res['month'] = 0;
      }
      
      $res['days'] = floor($secs / 86400);
      $secs = $secs % 86400;
      
      if(! $res['month']) {
        $res['hours'] = floor($secs / 3600);
        $secs = $secs % 3600;
       
        $res['minutes'] = floor($secs / 60);
        $res['secs'] = $secs % 60;
      }
      else {
        $res['hours'] = 0;
        $res['minutes'] = 0;
      }
      
      return $res;
    }

}
