<?php
/**
* @Copyright ((c) plasma-web.ru
        * @license    GPLv2 or later
        */

namespace PPF3;

\defined('PPF3_VERSION') or die;

class projectModel extends baseModel
{
    public function __construct()
    {
        $this->closelink = 'projects';
        $this->savelink = 'projects&task=project.edit';
        $this->savecreatelink = 'projects&task=project.edit&id=0';
    }

    public function getItems()
    {
        $task = ppf::gettask();
        $id = (int) ppf::get('id', $_GET);
        $ses = ppf_session::get('ppf3');

        if ($task != 'project.edit') {
            return;
        }

        if ($id === 0) {
            $data = (object) ppf::get('project.savecreate', $ses);
            if (! isset($data->title)) {
                return;
            }
            $data->id = 0;
            $data->title .= ' copy';
        } else {
            $db = \JFactory::getDbo();
            $db->setQuery("SELECT * FROM #__ppf3_projects WHERE id = " .(int) $id);
            $data = $db->loadObject();
            $_SESSION['ppf3']['project.savecreate'] =  false;
        }

        $params = json_decode($data->params);
        if ($params) {
            foreach ($params as $name => $value) {
                $dname = 'p]['.$name;
                $data->$dname = $value;
            }
        }
        unset($data->params);
        return $data;
    }

    public function save()
    {
        $db = \JFactory::getDbo();

        $data = filter_input(INPUT_POST, 'ppffield', FILTER_DEFAULT , FILTER_REQUIRE_ARRAY);

        if (! $data['title']) {
            $this->setmessages('The title is not filled in.', 'err');
            return (int) $data['id'];
        }
        
        if ($data['categories']) {
            $data['categories'] = implode(',', $data['categories']);
        }
        
        if((int) $data['id']) {
          $path = ppf::getfolder((int) $data['id']);
          if ($this->folder_exist($path)) {
            $includes = new \FilesystemIterator($path, \FilesystemIterator::KEY_AS_FILENAME);
            foreach ($includes as $include) {
                if (! is_dir($include)) {
                    $data['p']['ppffile'] = $includes->key();
                    break;
                }
            }
          }
        }


        $datap = json_encode($data['p'], JSON_UNESCAPED_UNICODE | JSON_UNESCAPED_SLASHES | JSON_NUMERIC_CHECK);

        $inputData = array(
            'title' 	=> '\''.addslashes(strip_tags($data['title'])).'\'',
            'id' 	=> (int) $data['id'],
            'published'    => (int) $data['published'],
            'access'   => (int) $data['access'],
            'rating'   => (int) $data['rating'],
            'price'   => (float) $data['price'],
            'currency'   => '\''.addslashes($data['currency']).'\'',
            'language'   => '\''.addslashes($data['language']).'\'',
            'categories'   => '\''.addslashes($data['categories']).'\'',
            'hits'   => (int) $data['hits'],
            'params'   => '\''.addslashes($datap).'\'',
         );

        if ($_REQUEST['task'] === 'project.savecreate') {
            $data['params'] = $datap;
            $_SESSION['ppf3']['project.savecreate'] =  $data;
        }

        if ($inputData['id']) {
            foreach ($inputData as $key => $value) {
                $updates[] = $key .' = '. $value;
            }
            $db->setQuery("UPDATE #__ppf3_projects SET ".implode(', ', $updates). " WHERE id=".(int) $inputData['id']);
            $db->execute();
            $id = $inputData['id'];
        } else {
            $db->setQuery("INSERT INTO #__ppf3_projects (".implode(',', array_keys($inputData)).") VALUES (".implode(',', array_values($inputData)).")");
            $db->execute();
            $id = $db->insertid();
        }
        if($id) {
          if (! $this->uploadfile($id)) {
              $this->setmessages('The file is not uploaded.', 'err');
              return $id;
          }
        }
        return $id;
    }
    
    protected function uploadfile($id) {
      $rfiles = $_FILES['ppffield'];
      if (! isset($rfiles['name']['p']['ppffile']) || ! $name = $rfiles['name']['p']['ppffile']) {
          return true;
      }
      
      if ($rfiles['error']['p']['ppffile']) {
          return false;
      }
      if ($rfiles['tmp_name']['p']['ppffile'] == 'none' || !is_uploaded_file($rfiles['tmp_name']['p']['ppffile'])) {
        return false;
      }
      if (mb_substr(trim($name), 0, 1, "UTF-8") == '.') {
        return false;
      }
      if(preg_replace('/[\/:*?"<>|+%!@]/', '', $name) != $name) {
        return false;
      }
      
      if (str_replace($this->extBlacklist(), '', strtolower($name)) != strtolower($name)) {
          $this->setmessages('Such a file cannot be uploaded: '.$name, 'err');
          return false;
      }
      
      $path = ppf::getfolder($id);
      
      if ($this->folder_exist($path)) {
          $this->recursiveRemoveDir($path);
      }

      if (! $this->folder_exist($path)) {
          if(! mkdir($path, 0777, true)) return false;
          if (! move_uploaded_file($rfiles['tmp_name']['p']['ppffile'], $path . $name)) {
              $this->setmessages('The file could not be moved to the specified directory.', 'err');
              return false;
          }
          if(! chmod($path . $name, 0600)) {
            $this->setmessages('Failed to assign file permissions.', 'err');
            return false;
          }
      }
      else {
        return false;
      }
      
      $db = \JFactory::getDbo();
      $db->setQuery("SELECT params FROM #__ppf3_projects WHERE id = " .(int) $id);
      $data = $db->loadResult();
      $params = json_decode($data);
      $params->ppffile = $name;
      $datap = json_encode($params, JSON_UNESCAPED_UNICODE | JSON_UNESCAPED_SLASHES | JSON_NUMERIC_CHECK);
      $db->setQuery("UPDATE #__ppf3_projects SET params = '".addslashes($datap)."' WHERE id=".(int) $id);
      
      if($db->execute()) return true;
      return false;
    }
    
    public function extBlacklist()
    {
        return array('.php', '.cgi', '.pl', '.fcgi', '.scgi', '.sql', '.phtml', '.asp', '.js', '.py', '.exe', '.htm', '.htaccess', '.htpasswd', '.ini', '.sh', '.log');
    }
    
    protected function recursiveRemoveDir($dir)
    {
        $includes = new \FilesystemIterator($dir);

        foreach ($includes as $include) {
            if (is_dir($include) && !is_link($include)) {
                $this->recursiveRemoveDir($include);
            } else {
                unlink($include);
            }
        }

        rmdir($dir);
    }
    
    protected function folder_exist($folder)
    {
        $path = realpath($folder);
        return ($path !== false and is_dir($path)) ? true : false;
    }

    public function delete($page)
    {
        $db = \JFactory::getDbo();
        $new_id = $this->checkcid();
        foreach($new_id as $id) {
          
          if(ppf::config()->delfiles) {
            $path = ppf::getfolder($id);
            
            if ($this->folder_exist($path)) {
                $this->recursiveRemoveDir($path);
            }
            if ($this->folder_exist($path)) {
              $this->setmessages('Such a folder cannot be removed: '.$path, 'err');
              return false;
            }
          }
          
          $db->setQuery( 'DELETE FROM #__ppf3_projects WHERE id = '.(int)$id);
          $db->execute();
          if(! $db->getAffectedRows()) {
            $this->setmessages('PPF_ERR_DATABASE', 'err');
            return false;
          }

        }
        $this->setmessages(sizeof($new_id).' '.'PPF_N_ITEMS_DELETED', 'mes');
    }


}
