/**
* @Copyright ((c) plasma-web.ru
        * @license    GPLv2 or later
        */


(() => {
  document.addEventListener("DOMContentLoaded", () => {
    document.querySelectorAll('.ppf3_player_inner video').forEach((video) => {
      ppf3Player.initiate(video);
    })
  });
  
  return ppf3Player = {
    initiate: function (video)
    {
      const box = video.closest('.ppf3player'),
      params = JSON.parse(box.dataset.strim),
      $ = (s,c) => (c ?? box).querySelector(s),
      player_range = $('.player_range'),
      player_progress = $('progress'),
      player_tooltip = $('.player_tooltip'),
      player_poster = $('.player_poster'),
      player_play = $('.player_play'),
      player_bottom = $('.player_bottom'),
      volume_range = $('.volume_range'),
      volume_button = $('.volume_button'),
      volume_icons = box.querySelectorAll('.rays div'),
      player_fullscreen = $('.player_fullscreen'),
      controls = $('.controls'),
      progressbar = $('.progressbar'),
      player_payline = $('.player_payline'),
      fm_video = $('.fm_video'),
      is_numeric = (str) => (str == '') ? false : !isNaN( str ),
      endPlay = () => {
        if(! video.paused) video.pause();
        if(video.currentTime) video.currentTime = 0;
        player_range.value = 0;
        player_poster.style.opacity = 1;
        player_play.style.opacity = 1;
      },
      formatTime = (seconds) => {
        const result = new Date(seconds * 1000).toISOString().substr(11, 8);
        if(result.substr(0, 2) == '00')
          return (result.substr(3, 1) == '0') ? result.substr(4, 4) : result.substr(3, 5);
        else return (result.substr(0, 1) == '0') ? result.substr(1, 6) : result.substr(0, 7);
      }
      
      let stop, timer1, timer2;
      
      video.controls = false;
      video.volume = 0.69;
      volume_button.innerHTML = this.volume_icon();
      player_fullscreen.innerHTML = this.fullscreen_icon();

      if(is_numeric(params.poster)) {
        const generateThumbnail = () => {
          video.removeEventListener('seeked', generateThumbnail);
          this.generateThumbnail(video);
        }
        video.addEventListener("seeked", generateThumbnail);
      }
      else {
        player_poster.innerHTML = '<img src="' + params.poster + '">';
        params.poster = 0;
      }

      video.addEventListener('loadedmetadata', () => {
        video.currentTime = params.poster * video.duration / 100;
        stop = params.pretime * video.duration / 100;
        
        const duration = Math.round(video.duration);
        $('.end').innerHTML = formatTime(duration);
        player_range.setAttribute('max', duration);
        player_progress.setAttribute('max', duration);
      });

      player_range.addEventListener('input', (e) => {
        // const skipTo = e.target.dataset.seek ? e.target.dataset.seek : e.target.value;
        const skipTo = e.target.value;
        if(skipTo >= stop) {
          return endPlay();
        }
        video.currentTime = skipTo;
        player_range.value = skipTo;
        player_progress.value = Math.floor(video.buffered.end(0));
        player_poster.style.opacity = 0;
        player_play.style.opacity = 0;
      });
      
      video.addEventListener('ended', () => {
        endPlay();
      });
      
      video.addEventListener('timeupdate', function () {
        $('.start').innerHTML = formatTime(this.currentTime);
        player_range.value = Math.floor(this.currentTime * 10) / 10;
        player_progress.value = Math.floor(this.buffered.end(0));
        if(player_payline) $('.timer',player_payline).innerHTML = formatTime(stop - this.currentTime);
        
        if(this.currentTime >= stop || this.currentTime == 0) {
          return endPlay();
        }
      });
      
      video.addEventListener('volumechange', () => {
        const v = video.volume,
        rem = (arr) => arr.forEach(i => {volume_icons[i].classList.remove('hidden')});
        
        volume_icons.forEach(icon => {icon.classList.add('hidden')});

        if (video.muted || v === 0) rem([0]);
        else if (v > 0 && v <= 0.3) rem([1]);
        else (v > 0.3 && v <= 0.7) ? rem([1,2]) : rem([1,2,3]);
      });
      
      
      player_play.onclick = function () {
        if(video.paused || video.ended) {
          player_poster.style.opacity = 0;
          this.style.opacity = 0;
          video.play();
        }
        else {
          this.style.opacity = 0.6;
          video.pause();
        }
      }
      
      player_range.onmousemove = (e) => {
        const skipTo = Math.round((e.offsetX / e.target.clientWidth) * parseInt(e.target.getAttribute('max'), 10));
        // e.target.setAttribute('data-seek', skipTo);
        player_tooltip.textContent = formatTime(skipTo);
        const rect = e.target.getBoundingClientRect();
        player_tooltip.style.left = `${e.pageX - rect.left}px`;
        player_tooltip.style.display = '';
      }
      
      player_range.onmouseout = (e) => {
        player_tooltip.style.display = 'none';
      }
      
      volume_range.oninput = (e) => {
          if (video.muted) video.muted = false;
          video.volume = e.target.value;
      }
      
      volume_button.onclick = (e) => {
        video.muted = ! video.muted;
        player_bottom.style.opacity = 1;
        
        if (video.muted) {
          volume_range.setAttribute('data-volume', volume_range.value);
          volume_range.value = 0;
        } else {
          volume_range.value = volume_range.dataset.volume;
        }
      }
      
      player_fullscreen.onclick = (e) => {
        if (document.fullscreenElement) {
          document.exitFullscreen();
        } else if (document.webkitFullscreenElement) {
          document.webkitExitFullscreen();
        } else if (box.webkitRequestFullscreen) {
          box.webkitRequestFullscreen();
        } else {
          box.requestFullscreen();
        }
      }
      
      controls.onmouseenter = (e) => {
        progressbar.style.opacity = 1;
        // player_bottom.style.opacity = 1;
      }
      
      controls.onmouseleave = (e) => {
        clearTimeout(timer1);
        player_bottom.style.opacity = 0;
        if(video.paused || video.ended){
          timer1 = setTimeout(()=>{progressbar.style.opacity = 0}, 1500);
        }
        else timer1 = setTimeout(()=>{progressbar.style.opacity = 0}, 300);
      }
      
      controls.onmousemove = (e) => {
        progressbar.style.opacity = 1;
        clearTimeout(timer1);
        timer1 = setTimeout(()=>{
          progressbar.style.opacity = 0;
          player_bottom.style.opacity = 0;
        }, 3000);
      }
      
      player_bottom.onmousemove = (e) => {
        player_bottom.style.opacity = 1;
        clearTimeout(timer2);
        timer2 = setTimeout(()=>{player_bottom.style.opacity = 0}, 2000);
      }
      
      if(fm_video) {
        $('.fullbtn').onclick = (e) => {
          fm_video.style.zIndex = 1;
          fm_video.style.opacity = 1;
          if(! video.paused) video.pause();
        }
        $('.ppf3_close').onclick = (e) => {
          setTimeout(()=>{fm_video.style.zIndex = 0}, 1000);
          fm_video.style.opacity = 0;
        }
      }
      
    },
    
    generateThumbnail: function (video) {
      const width = video.videoWidth,
      height = video.videoHeight,
      canvas = document.createElement('canvas');
      
      canvas.width = width;
      canvas.height = height;
      
      const context = canvas.getContext('2d');
      context.drawImage(video, 0, 0, width, height);
      
      const poster = video.closest('.ppf3_player_inner').querySelector('.player_poster');
      poster.innerHTML = '<img src="' + canvas.toDataURL() + '">';
      video.currentTime = 0;
      video.pause();
    },
    
    volume_icon: function () {
      return '<svg version="1.1" viewBox="0 0 91 91" xml:space="preserve"><g><path d="M15.464,30.723c0.343-0.037,0.684-0.082,1.027-0.122c-0.474,4.487-0.731,9.004-0.938,13.504   c-0.304,6.73-1.351,13.971-0.8,20.687c0.494,6.028,5.519,6.497,10.482,6.553c3.364,0.037,12.804,0.042,14.024-0.288   c10.561,7.425,22.968,11.636,33.39,19.305c2.448,1.802,5.977-0.634,5.925-3.402c-0.542-27.744-1.77-55.509-1.173-83.264   c0.05-2.344-2.716-3.956-4.72-2.708C62.019,7.613,51.23,13.984,41.204,21.522c-0.928-1.161-2.354-1.909-4.59-1.947   c-6.953-0.12-14.195,1.115-21.15,1.535C9.234,21.487,9.405,31.41,15.464,30.723z M70.819,9.195   c-1.141,23.427,0.07,46.825-0.04,70.26C61.89,73.75,43.58,62.549,43.431,62.449c0.055-3.696,0.079-7.39,0.067-11.086   c-0.021-6.914-0.182-13.829-0.453-20.737c-0.069-1.762-0.091-3.644-0.371-5.348C51.762,19.45,61.354,14.396,70.819,9.195z    M34.465,28.542c0.344,3.244,0.405,18.5,0.467,22.821c0.025,1.729-0.412,9.705-0.055,11.596   c-1.837-0.357-12.466,0.127-12.583,0.116c-0.053-0.925-1.363-26.876-1.984-32.928C22.481,29.89,33.496,28.632,34.465,28.542z"/></g></svg>';
    },
    
    fullscreen_icon: function () {
      return '<svg version="1.1" viewBox="0 0 489.001 489.001" xml:space="preserve"><g><path d="M2.2,168.151l-2.1-151.3c-0.1-7.7,6.2-14,13.9-13.9l151.3,2.2c12.1,0.2,18.1,14.8,9.5,23.4l-42.1,42.1l70,70l-65,65l-70-70l-42.1,42.1C17.1,186.251,2.4,180.251,2.2,168.151z"/><path d="M421.3,136.551l42.1,42.1c8.6,8.6,23.2,2.6,23.4-9.5l2.2-151.3c0.1-7.7-6.2-14-13.9-13.9l-151.3,2.2c-12.1,0.2-18.1,14.8-9.5,23.4l42,41.9l-70,70l65,65L421.3,136.551z"/><path d="M314.2,460.451c-8.6,8.6-2.6,23.2,9.5,23.4l151.3,2.2c7.7,0.1,14-6.2,13.9-13.9l-2.2-151.3c-0.2-12.1-14.8-18.1-23.4-9.5l-42.1,42.1l-70-70l-65,65l70,70L314.2,460.451z"/><path d="M14,485.051l151.3-2.2c12.1-0.2,18.1-14.8,9.5-23.4l-42.1-42l70-70l-65-65l-70,70l-42.1-42.1c-8.6-8.6-23.2-2.6-23.4,9.5L0,471.151C0,478.851,6.3,485.151,14,485.051z"/></g></svg>';
    }
    
  }
})();