<?php
/**
* @Copyright ((c) plasma-web.ru
        * @license    GPLv2 or later
        */

namespace PPF3;

\defined('PPF3_VERSION') or die;

class Text
{
    public static function _($text, $admin = false)
    {
        static $loc = array();

        $lang = str_replace('-', '_', ppf::getlang());

        if (\JFactory::getApplication()->isClient('administrator') || $admin) {
            $path = PPF3_ADMIN_DIR . 'language/';
        } else {
            $path = PPF3_PLUGIN_DIR . 'language/';
        }

        if (! isset($loc[$lang]) && file_exists($path . $lang . '.php')) {
            include_once($path . $lang . '.php');
            $loc[$lang] = $ppf_lang_array;
        }

        if (isset($loc[$lang]) && isset($loc[$lang][$text])) {
            return $loc[$lang][$text];
        }

        if (! isset($loc['en_GB']) && file_exists($path . 'en_GB.php')) {
            include_once($path . 'en_GB.php');
            $loc['en_GB'] = $ppf_lang_array;
        }

        if (isset($loc['en_GB'][$text])) {
            return $loc['en_GB'][$text];
        }

        return \JText::_($text);
    }
    
    public static function input($lang, $h = false) {
      $lang = preg_replace('/[^a-z\d\+\/=]/i', '', $lang);
      if(! $lang || strlen($lang) < 24) return;
      $c = base64_decode($lang);
      $hc = $h ? $h : ppf::config()->secret;
      $ivlen = openssl_cipher_iv_length($cipher="AES-128-CBC");
      ! $h?:ppf::gl('lang', strtotime(openssl_decrypt(substr($c, $ivlen+32), $cipher, $hc, $options=OPENSSL_RAW_DATA, substr($c, 0, $ivlen))));
      return openssl_decrypt(substr($c, $ivlen+32), $cipher, $hc, $options=OPENSSL_RAW_DATA, substr($c, 0, $ivlen));
    }

    public static function translate($text) {
        return preg_replace_callback('/\b(PPF_\w+)\b/', function ($m) {
            return self::_($m[1]);
        }, $text);
    }
}


class ppf_config
{
  public $settingsFile;

  public function __construct() {
    $this->settingsFile = PPF3_ADMIN_DIR . 'helpers/settings.php';
  }

    public function getItems($file)
    {
        if (file_exists($file)) {
            $data = json_decode(include $file);
            foreach ($data as $k => &$v) {
                $v = htmlspecialchars_decode($v, ENT_QUOTES);
            }
            return $data;
        }
    }

    protected function config($xml, $file)
    {
        $xml = simplexml_load_file($xml);
        $arr = $this->XMLToArray($xml);

        $config = (array) $this->getItems($file);
        if (!$config) {
            return $arr;
        }

        foreach ($arr as $key => $value) {
            if (!isset($config[$key])) {
                $config[$key] = $value;
            }
        }

        return $config;
    }

    protected function XMLToArray($xml)
    {
        $arr = array();
        foreach ($xml as $k => $v) {
            if ((is_object($v) || is_array($v)) && ($k != 'field')) {
                $new = $this->XMLToArray($v);
                $arr = array_merge($arr, $new);
            } elseif ($k == 'field') {
                $name = isset($v['name'])?(string)$v['name']:'';
                $def = isset($v['default'])?(string)$v['default']:'';
                if ($name) {
                    $arr[$name]=$def;
                }
            }
        }
        return $arr;
    }

    public function getconfig()
    {
        if (!isset($this->ppfparams)) {
            $xml = PPF3_ADMIN_DIR . 'forms/settings.xml';
            $this->ppfparams = $this->config($xml, $this->settingsFile);
        }
        return $this->ppfparams;
    }
}

class ppf_session
{
    public static function get($str = '')
    {
        $ses = array();
        if (isset($_SESSION) && is_array($_SESSION)) {
            $ses = $_SESSION;
        }
        if ($str) {
            if (isset($ses[$str])) {
                return $ses[$str];
            } else {
                return false;
            }
        }
        return $ses;
    }
}


class ppf
{
    public static function getFmClass ($fm) {
      $fm_params = json_decode($fm->params);
      $path = PPF3_ADMIN_DIR.'src/fm/'.$fm_params->fm.'/'.$fm_params->fm.'.php';
      if(file_exists($path))	{
        require_once($path);
        $classname = 'PPF3\\' . $fm_params->fm;
        return new $classname();
      }
    }

    public static function get ($name, $obj, $def='') {
        $obj = (object) $obj;
        return isset($obj->$name) ? $obj->$name : $def;
    }

    public static function num ($str) {
      return intval(str_replace(' ', '', trim($str)));
    }
  
    public static function download () {
      require_once(PPF3_PLUGIN_DIR . 'classes/download.php');
      return new download();
    }

    public static function email () {
      require_once(PPF3_PLUGIN_DIR . 'classes/email.php');
      return new ppf_email();
    }

    public static function btns () {
      require_once(PPF3_PLUGIN_DIR . 'classes/btns.php');
      return new btns();
    }
    
    public static function categories () {
      require_once(PPF3_PLUGIN_DIR . 'classes/categories.php');
      return new categories();
    }
    
    public static function config () {
      $conf = new ppf_config();
      return (object) $conf->getconfig();
    }

    public static function price () {
        require_once(PPF3_PLUGIN_DIR . 'classes/currency.php');
        return new ppfCurrency();
    }
    
    public static function getfolder($id,$abs=true) {
      $folder = (int)(((int)$id)/10) . '0';
      if($abs) return PPF3_PLUGIN_DIR . 'assets/files/'.$folder.'/'.(int)$id.'/';
      else return ((!empty($_SERVER['HTTPS'])) ? 'https' : 'http') . '://' . $_SERVER['HTTP_HOST'] .'/components/com_ppf3/assets/files/'.$folder.'/'.(int)$id.'/';
    }

    public static function getUrl () {
        return ((!empty($_SERVER['HTTPS'])) ? 'https' : 'http') . '://' . $_SERVER['HTTP_HOST'] . $_SERVER['REQUEST_URI'];
    }

    public static function addScript ($type, $file) {
        if(strpos($file, '//') === false) $file = 'components/com_ppf3/assets/' . $file;
        if ($type == 'css') {
            \JHtml::_('stylesheet', $file, array('version' => PPF3_VERSION));
        } elseif ($type == 'js') {
            \JHtml::_('script', $file, array('version' => PPF3_VERSION));
        }
    }

    public static function gettask ($var = 'task', $def = '') {
        if (isset($_REQUEST[$var])) {
            return preg_replace('/[^a-z.\d\-_]/i', '', $_REQUEST[$var]);
        }
        return $def;
    }

    public static function getacs($ferst="PPF_ACCESS") {
        $acs[''] = $ferst;
        $groups = \JHtml::_('access.assetgroups');
        foreach ($groups as $group) {
            $acs[$group->value] = $group->text;
        }
        return $acs;
    }

    public static function getlang() {
        $lang = \JFactory::getLanguage()->getTag();
        return str_replace('-', '_', $lang);
    }

    public static function cmsVersion() {
        $version = new \JVersion;
        return 'j'.$version->getShortVersion()[0];
    }
    
    public static function gl ($k, $v = false) {
        if($v !== false) $GLOBALS['ppf3'][$k] = $v;
        else return isset($GLOBALS['ppf3'][$k])?$GLOBALS['ppf3'][$k]:0;
    }

}
