<?php
/**
* @Copyright ((c) plasma-web.ru
        * @license    GPLv2 or later
        */

namespace PPF3;

\defined('PPF3_VERSION') or die;

class download
{
  public function loadfile ($id)
  {
    $fm = ppf::btns()->getFm($id);
    if(! $fm) return $this->error();
    
    $fm_params = json_decode($fm->params);
    $speed = 1024 * (int) ppf::get('speed', $fm_params, 0);
    
    $accept = (int) ppf::btns()->getAccess($fm);
    if(! $accept) return $this->error();

    $fileName = ppf::get('ppffile', $fm_params, 'empty');

    $file = ppf::getfolder($id) . $fileName;
    if(! is_file($file)) return $this->error();
    
    // ini_set('implicit_flush', 1);

    if(! $speed) $buffer = 1024*1024;
    else $buffer = $speed;
    
    $filesize = filesize($file);
    if($buffer > $filesize) $buffer = $filesize;

    $f = fopen($file, 'rb');
    
    $expl = explode(".", $file);
    header('Content-Type: ' . $this->type(end($expl)));

    $start = 0;
    $length = $filesize;

    if(! isset($_SERVER['HTTP_RANGE'])) {
      header($_SERVER['SERVER_PROTOCOL'].' 200 OK');
    }
    else {
      preg_match('/bytes=(\d+)-(\d+)?/', $_SERVER['HTTP_RANGE'], $matches);
      
      $start = intval($matches[1]);
      $end = $start + 2*$buffer;
      
      if($end > ($filesize - 1)) $end = $filesize - 1;
      $length = $end + 1 - $start;
      
      header($_SERVER['SERVER_PROTOCOL'].' 206 Partial Content');
      header("Content-Range: bytes $start-$end/$filesize");
    }
    
    header('Accept-Ranges: bytes');
    header('Content-Length: ' . $length);
    header("Content-Disposition: attachment; filename=\"$fileName\"");
    header('Connection: close');
    header("Cache-Control: no-cache, must-revalidate");
    header('X-Accel-Buffering: no');

    fseek($f, $start);
    
    if(! $start && ppf::get('fm', $fm_params) == 'fm_free') {
      $this->freestat($fm);
    }
    
    while (ob_get_level() > 0) {
        ob_end_flush();
    }
    ob_implicit_flush(true);
    
    while ($length >= $buffer)
    {
      echo fread($f, $buffer);
      $length -= $buffer;
      if ($speed) sleep(1);
    }
    if ($length > 0) echo fread($f, $length);
    fclose($f);
    exit;
  }
  
  
  
  
  public function loadvideo($id) {
    $fm = ppf::btns()->getFm($id);
    
    if(! $fm) return $this->error();
    $fm_params = json_decode($fm->params);
    
    $file = ppf::get('url', $fm_params);
    $pats = explode($_SERVER['HTTP_HOST'], $file);
    $file = JPATH_ROOT . $pats[1];
    if(! is_file($file)) return $this->error();
        
    $presize = ppf::num(ppf::get('pretime', $fm_params, 0)) * 1024;
    $speed = 1024 * (int) ppf::get('speed', $fm_params, 0);
    
    $accept = (int) ppf::btns()->getAccess($fm);
    if($accept) $presize = 0;
    elseif(! $presize) return;
    
    if(! $speed) $buffer = 1024*1024;
    else $buffer = $speed;

    $filesize = filesize($file);
    $f = fopen($file, 'rb');
    
    if($presize && ($filesize > $presize)) $filesize = $presize;
    if($buffer > $filesize) $buffer = $filesize;
    
    $expl = explode(".", $file);
    header('Content-Type: ' . $this->type(end($expl)));

    $start = 0;
    $length = $filesize;
    
    if(! isset($_SERVER['HTTP_RANGE'])) {
      header($_SERVER['SERVER_PROTOCOL'].' 200 OK');
    }
    else {
      preg_match('/bytes=(\d+)-(\d+)?/', $_SERVER['HTTP_RANGE'], $matches);
      
      $start = intval($matches[1]);
      $end = $start + 2*$buffer;
      
      if($end > ($filesize - 1)) $end = $filesize - 1;
      $length = $end + 1 - $start;
      
      header($_SERVER['SERVER_PROTOCOL'].' 206 Partial Content');
      header("Content-Range: bytes $start-$end/$filesize");
    }
    
    header('Accept-Ranges: bytes');
    header('Content-Length: ' . $length);
    header("Content-Disposition: inline");
    header('Connection: close');

    fseek($f, $start);
    
    while ($length >= $buffer)
    {
      echo fread($f, $buffer);
      $length -= $buffer;
      if ($speed) sleep(1);
    }
    if ($length > 0) echo fread($f, $length);
    fclose($f);
    exit;
  }
  
  
  
  public function freestat($fm) {
    $cook = 'fstat_'.$fm->id;
    if (isset($_COOKIE[$cook])) return;
    
    $db = \JFactory::getDbo();
    $db->setQuery("UPDATE #__ppf3_projects SET hits = ( hits + 1 ) WHERE id = ".(int) $fm->id);
    $db->execute();
    
    $now = date('ymd', time());
    $last = $now . '=1,';
    $path = PPF3_PLUGIN_DIR . 'assets/freestat/'.$fm->id.'.ini';
    
    if(is_file($path)) {
      $str = file_get_contents($path);
      $pos = strpos($str, $now.'=');
      if($pos !== false) {
        $new = $now . '=' . (1 + (int) substr($str, $pos+7)) . ',';
        $str = str_replace(substr($str, $pos), $new, $str);
        file_put_contents($path, $str);
      }
      else file_put_contents($path, $last, FILE_APPEND);
    }
    else file_put_contents($path, $last);
    
    setcookie($cook, 1, time() + 24*3600, '/');
  }
  
    public function error()
    {
        header('Content-Type: application/octet-stream');
        header('Content-Disposition: attachment; filename="error.txt";');
        echo 'there is no such file on our server';
    }
    
    protected function type ( $ext ) {
        switch ( $ext ) {
             case "pdf": return "application/pdf";
             case "zip": return "application/zip";
             case "rar": return "application/rar";
             case "doc":
             case "docx": return "application/msword";
             case "xls": return "application/vnd.ms-excel";
             case "ppt": return "application/vnd.ms-powerpoint";
             case "gif": return "image/gif";
             case "png": return "image/png";
             case "jpeg":
             case "jpg": return "image/jpg";
             case "mp3": return "audio/mpeg";
             case "mp4":
             case "mpeg":
             case "mpg":
             case "mpe": return "video/mpeg";
             case "mov":
             case "swf":
             case "3gp":
             case "m4a":
             case "aac":
             case "m3u": return "video/quicktime";
             case "wav":
             case "wmv":
             case "asf":
             case "divx":
             case "avi": return "video/x-msvideo";
             case "ra":
             case "ram":
             case "ogg": return "audio/x-pn-realaudio";
  
             case "php":
             case "htm":
             case "html": die("<b>Cannot be used for ". $ext ." files!</b>");
    
             default: return "application/octet-stream";
        }
      }

}